import {AppStateModel} from '@app/app.state';
import {
  Create,
  CreateFail,
  CreateSuccess,
  GetAll,
  GetAllFail,
  GetAllSuccess,
  GetById,
  GetByIdFail,
  GetByIdSuccess,
} from '@app/deposit/deposit.action';
import {RoutesEnum} from '@app/enums/routes.enum';
import {StateEnum} from '@app/enums/state.enum';
import {DepositsModel, DipModel} from '@app/generated-api';
import {CollectionTypedModel} from '@app/shared/models/collection-typed.model';
import {DlcmExtendedService} from '@app/service/dlcm.extended.service';
import {Navigate} from '@ngxs/router-plugin';
import {Action, State, StateContext} from '@ngxs/store';
import {catchError} from 'rxjs/internal/operators/catchError';
import {tap} from 'rxjs/operators';

export interface DepositStateModel extends AppStateModel {
  totalDeposit: number;
  deposits: DipModel[];
  currentDeposit: DipModel;
}

@State<DepositStateModel>({
  name: StateEnum.deposit,
  defaults: {
    totalDeposit: 0,
    deposits: [],
    isLoading: false,
    currentDeposit: null,
  },
})
export class DepositState {
  constructor(private dlcmService: DlcmExtendedService) {
  }

  @Action(GetAll, {cancelUncompleted: true})
  getAll(ctx: StateContext<DepositStateModel>, action: GetAll) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.preingestDepositsGet()
      .pipe(
        tap((collectionDeposits: CollectionTypedModel<DepositsModel>) => {
          ctx.dispatch(new GetAllSuccess(collectionDeposits));
        }),
        catchError(error => {
          ctx.dispatch(new GetAllFail());
          throw error;
        }),
      );
  }

  @Action(GetAllSuccess)
  getAllSuccess(ctx: StateContext<DepositStateModel>, action: GetAllSuccess) {
    ctx.patchState({
      deposits: action.deposits._data,
      totalDeposit: action.deposits._page.totalItems,
      isLoading: false,
    });
  }

  @Action(GetAllFail)
  getAllFail(ctx: StateContext<DepositStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }

  @Action(GetById, {cancelUncompleted: true})
  getById(ctx: StateContext<DepositStateModel>, action: GetById) {
    ctx.patchState({
      isLoading: true,
      currentDeposit: null,
    });

    return this.dlcmService.preingestDepositsGetById(action.id)
      .pipe(
        tap((deposit: DepositsModel) => {
          ctx.dispatch(new GetByIdSuccess(deposit));
        }),
        catchError(error => {
          ctx.dispatch(new GetByIdFail());
          throw error;
        }),
      );
  }

  @Action(GetByIdSuccess)
  getByIdSuccess(ctx: StateContext<DepositStateModel>, action: GetByIdSuccess) {
    ctx.patchState({
      currentDeposit: action.deposit,
      isLoading: false,
    });
  }

  @Action(GetByIdFail)
  getByIdFail(ctx: StateContext<DepositStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }


  @Action(Create, {cancelUncompleted: true})
  create(ctx: StateContext<DepositStateModel>, action: Create) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.preingestDepositsPost(action.deposit)
      .pipe(
        tap(() => {
          ctx.dispatch(new CreateSuccess());
        }),
        catchError(error => {
          ctx.dispatch(new CreateFail());
          throw error;
        }),
      );
  }

  @Action(CreateSuccess)
  createSuccess(ctx: StateContext<DepositStateModel>) {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([RoutesEnum.deposit]));
  }

  @Action(CreateFail)
  createFail(ctx: StateContext<DepositStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }
}



