import {AppStateModel} from '@app/app.state';
import {RoutesEnum} from '@app/enums/routes.enum';
import {StateEnum} from '@app/enums/state.enum';
import {OrganizationalUnitsModel} from '@app/generated-api';
import {CollectionTypedModel} from '@app/shared/models/collection-typed.model';
import {
  Create,
  CreateFail,
  CreateSuccess,
  Delete,
  DeleteFail,
  DeleteSuccess,
  GetAll,
  GetAllFail,
  GetAllSuccess,
  GetById,
  GetByIdFail,
  GetByIdSuccess,
  Update,
  UpdateFail,
  UpdateSuccess,
} from '@app/organizational-unit/organizational-unit.action';
import {DlcmExtendedService} from '@app/service/dlcm.extended.service';
import {Navigate} from '@ngxs/router-plugin';
import {Action, State, StateContext} from '@ngxs/store';
import {catchError} from 'rxjs/internal/operators/catchError';
import {tap} from 'rxjs/operators';

export interface OrganizationalUnitStateModel extends AppStateModel {
  totalOrganizationalUnits: number;
  organizationalUnits: OrganizationalUnitsModel[];
  currentOrganizationalUnit: OrganizationalUnitsModel,
}

@State<OrganizationalUnitStateModel>({
  name: StateEnum.organizationalUnit,
  defaults: {
    totalOrganizationalUnits: 0,
    isLoading: false,
    organizationalUnits: [],
    currentOrganizationalUnit: null,
  },
})
export class OrganizationalUnitState {
  constructor(private dlcmService: DlcmExtendedService) {
  }

  @Action(GetAll, {cancelUncompleted: true})
  getAll(ctx: StateContext<OrganizationalUnitStateModel>, action: GetAll) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.adminOrganizationalUnitsGet()
      .pipe(
        tap((collectionDeposits: CollectionTypedModel<OrganizationalUnitsModel>) => {
          ctx.dispatch(new GetAllSuccess(collectionDeposits));
        }),
        catchError(error => {
          ctx.dispatch(new GetAllFail());
          throw error;
        }),
      );
  }

  @Action(GetAllSuccess)
  getAllSuccess(ctx: StateContext<OrganizationalUnitStateModel>, action: GetAllSuccess) {
    ctx.patchState({
      organizationalUnits: action.organizationalUnits._data,
      totalOrganizationalUnits: action.organizationalUnits._page.totalItems,
      isLoading: false,
    });
  }

  @Action(GetAllFail)
  getAllFail(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }

  @Action(GetById, {cancelUncompleted: true})
  getById(ctx: StateContext<OrganizationalUnitStateModel>, action: GetById) {
    ctx.patchState({
      isLoading: true,
      currentOrganizationalUnit: null,
    });

    return this.dlcmService.adminOrganizationalUnitsGetById(action.id)
      .pipe(
        tap((deposit: OrganizationalUnitsModel) => {
          ctx.dispatch(new GetByIdSuccess(deposit));
        }),
        catchError(error => {
          ctx.dispatch(new GetByIdFail());
          throw error;
        }),
      );
  }

  @Action(GetByIdSuccess)
  getByIdSuccess(ctx: StateContext<OrganizationalUnitStateModel>, action: GetByIdSuccess) {
    ctx.patchState({
      currentOrganizationalUnit: action.organizationalUnit,
      isLoading: false,
    });
  }

  @Action(GetByIdFail)
  getByIdFail(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }


  @Action(Create, {cancelUncompleted: true})
  create(ctx: StateContext<OrganizationalUnitStateModel>, action: Create) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.adminOrganizationalUnitsPost(action.organizationalUnit)
      .pipe(
        tap(() => {
          ctx.dispatch(new CreateSuccess());
        }),
        catchError(error => {
          ctx.dispatch(new CreateFail());
          throw error;
        }),
      );
  }

  @Action(CreateSuccess)
  createSuccess(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([RoutesEnum.organizationalUnit]));
  }

  @Action(CreateFail)
  createFail(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }


  @Action(Update, {cancelUncompleted: true})
  update(ctx: StateContext<OrganizationalUnitStateModel>, action: Update) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.adminOrganizationalUnitsPatchById(action.organizationalUnit.resId, action.organizationalUnit)
      .pipe(
        tap(() => {
          ctx.dispatch(new UpdateSuccess());
        }),
        catchError(error => {
          ctx.dispatch(new UpdateFail());
          throw error;
        }),
      );
  }

  @Action(UpdateSuccess)
  updateSuccess(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([RoutesEnum.organizationalUnit]));
  }

  @Action(UpdateFail)
  updateFail(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }

  @Action(Delete, {cancelUncompleted: true})
  delete(ctx: StateContext<OrganizationalUnitStateModel>, action: Delete) {
    ctx.patchState({
      isLoading: true,
    });

    return this.dlcmService.adminOrganizationalUnitsDeleteById(action.resId)
      .pipe(
        tap(() => {
          ctx.dispatch(new DeleteSuccess());
        }),
        catchError(error => {
          ctx.dispatch(new DeleteFail());
          throw error;
        }),
      );
  }

  @Action(DeleteSuccess)
  deleteSuccess(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([RoutesEnum.organizationalUnit]));
  }

  @Action(DeleteFail)
  deleteFail(ctx: StateContext<OrganizationalUnitStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }
}
